// bmptext.cpp
// by Glenn G. Chappell
// September 2003
//
// For CS 381
// GLUT Bitmap Text Demo

#include <iostream>
using std::cerr;
using std::endl;
#include <string>
using std::string;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff, includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400; // Starting window width & height, in pixels

// Keyboard
const int ESCKEY = 27;        // ASCII value of escape character

// Globals affecting display
bool showtext = true;         // True if text (help) is shown
int textcolor = 0;            // value 0..numtextcolors-1, indicating text color
const int numtextcolors = 4;
int bkgndcolor = 0;           // value 0..numbkgndcolors-1, indicating background color
const int numbkgndcolors = 3;


// printbitmap
// Prints the given string at the given raster position
//  using GLUT bitmap fonts.
// You probably don't want any rotations in the model/view
//  transformation when calling this function.
void printbitmap(const string msg, double x, double y)
{
   glRasterPos2d(x, y);
   for (string::const_iterator ii = msg.begin();
        ii != msg.end();
        ++ii)
   {
      glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *ii);
   }
}


// display
// The GLUT display function
void display()
{
   // Set background color
   switch (bkgndcolor)
   {
   case 0:
      glClearColor(1.0, 1.0, 1.0, 0.0);  // white
      break;
   case 1:
      glClearColor(0.9, 0.9, 0.1, 0.0);  // yellow
      break;
   case 2:
      glClearColor(0.0, 0.0, 0.0, 0.0);  // black
      break;
   }

   glClear(GL_COLOR_BUFFER_BIT);

   // Set text color
   switch (textcolor)
   {
   case 0:
      glColor3d(0.1, 0.1, 0.9);  // blue
      break;
   case 1:
      glColor3d(0.9, 0.1, 0.1);  // red
      break;
   case 2:
      glColor3d(0.1, 0.9, 0.1);  // green
      break;
   case 3:
      glColor3d(0.7, 0.1, 0.9);  // purple
   }

   // Draw text
   if (showtext)
   {
      printbitmap("GLUT Bitmap Text Demo", 0.05, 0.95);
      printbitmap("H     Toggle help", 0.05, 0.85);  // Blank line between previous & this
      printbitmap("T     Cycle text color", 0.05, 0.8);
      printbitmap("B     Cycle background color", 0.05, 0.75);
      printbitmap("Esc   Quit", 0.05, 0.7);
   }

   glutSwapBuffers();
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }
}


// keyboard
// The GLUT keyboard function
void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
   case ESCKEY:  // ESC: Quit
      exit(0);
      break;
   case 'h':  // h - toggle text (help)
   case 'H':  // upper- and lower-case are different!
      showtext = !showtext;
      glutPostRedisplay();
      break;
   case 't':  // t - cycle text color
   case 'T':
      ++textcolor;
      textcolor = textcolor % numtextcolors;
      glutPostRedisplay();
      break;
   case 'b':  // b - cycle background color
   case 'B':
      ++bkgndcolor;
      bkgndcolor = bkgndcolor % numbkgndcolors;
      glutPostRedisplay();
      break;
   }
}


// init
// Initializes GL states
// Called by main after window creation
void init()
{
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluOrtho2D(0.0, 1.0, 0.0, 1.0);

   glMatrixMode(GL_MODELVIEW);  // Always go back to model/view mode
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("CS 381 - GLUT Bitmap Text Demo");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display); 
   glutIdleFunc(idle);
   glutKeyboardFunc(keyboard);

   // Do something
   glutMainLoop();

   return 0;
}
